/* Includes ------------------------------------------------------------------*/
#include "TouchPanel.h"
#include "spi.h"


/* Private define ------------------------------------------------------------*/
#define THRESHOLD 		2
#define TP_hspi 		hspi4

/*******************************************************************************
* Function Name  : TP_Init
* Description    :
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void TP_Init(void)
{
}

/*******************************************************************************
* Function Name  : DelayUS
* Description    :
* Input          : - cnt:
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void DelayUS(uint32_t cnt)
{
	volatile uint32_t i;
	i = cnt * 10;
	while(i--);
}


/*******************************************************************************
* Function Name  : WR_CMD
* Description    :
* Input          : - cmd:
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
static void WR_CMD (uint8_t cmd)
{
	HAL_SPI_Transmit(&TP_hspi, &cmd, 1, 1000);
}


/*******************************************************************************
* Function Name  : RD_AD
* Description    :
* Input          : None
* Output         : None
* Return         :
* Attention		 : None
*******************************************************************************/
static uint16_t RD_AD(void)
{
	uint8_t buf[2] = {0, 0 };
	int value;

	HAL_SPI_Receive(&TP_hspi, buf, 2, 1000);
	value = (((uint16_t)buf[0] * 256 +  buf[1]) >> 3) & 0x0FFF;

	return value;
}


/*******************************************************************************
* Function Name  : Read_X
* Description    : Read ADS7843 ADC X
* Input          : None
* Output         : None
* Return         :
* Attention		 : None
*******************************************************************************/
uint16_t Read_X(void)
{
  uint16_t i;

  TP_CS(0);
  DelayUS(20);
  WR_CMD(CHX);
  DelayUS(100);
  i = RD_AD();
  TP_CS(1);

  return i;
}

/*******************************************************************************
* Function Name  : Read_Y
* Description    : Read ADS7843 ADC Y
* Input          : None
* Output         : None
* Return         :
* Attention		 : None
*******************************************************************************/
uint16_t Read_Y(void)
{
	uint16_t i;

  TP_CS(0);
  DelayUS(20);
  WR_CMD(CHY);
  DelayUS(100);
  i = RD_AD();
  TP_CS(1);

  return i;
}


/*******************************************************************************
* Function Name  : TP_GetXY
* Description    : Read ADS7843
* Input          : None
* Output         : None
* Return         :
* Attention		 : None
*******************************************************************************/
void TP_GetXY(uint16_t *x, uint16_t *y)
{
	uint16_t tpx, tpy;

  tpx = Read_X();
  DelayUS(1);
  tpy = Read_Y();
  *x = tpx;
  *y = tpy;
}


// 좌표값은 그리 정밀하지 않습니다.
// 아래 수치는 터치 스크린의 가장자리에서 읽은 그대로의 값으로 제품마다 편차가 있습니다.
// 더 정밀한 좌표가 필요하면 캘리브레이션 과정이 들어가야 합니다.

#define MIN_X		120
#define MAX_X		3940
#define MIN_Y		300
#define MAX_Y		3900

int GetTouchStatus(Point * pos)
{
	uint16_t x, y;

	if(GPIO_PIN_SET == TP_INT_IN()) return 0;

	TP_GetXY(&y, &x);

	if((x > MAX_X) || (x < MIN_X)) return 0;
	if((y > MAX_Y) || (y < MIN_Y)) return 0;

	pos->X = 799 - (x - MIN_X) * 800 / (MAX_X - MIN_X);
	pos->Y = (y - MIN_Y) * 480 / (MAX_Y - MIN_Y);

	return 1;
}

int GetTouchPannel(uint16_t *x, uint16_t *y)
{
	Point pos;
	int i;

	HAL_Delay(5);
	if(GetTouchStatus(&pos) == 0) return 0;
	HAL_Delay(10);
	if(GetTouchStatus(&pos) == 0) return 0;
	HAL_Delay(15);
	if(GetTouchStatus(&pos) == 0) return 0;
	*x = pos.X;
	*y = pos.Y;

	for(i = 0; i < 20; i++)
	{
		HAL_Delay(10);
		if(GPIO_PIN_SET == TP_INT_IN()) break;
	}
	return 1;
}


/*********************************************************************************************************
      END FILE
*********************************************************************************************************/
